/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 10.05.2004
 * Time: 19:06:24
 */

/**
 * Provides encoding and decoding functions to map a number to a wavelength and vice versa.
 */
public class Coding {
    /**
     * The constructor.
     * @param n  the number of elements in the input
     * @param m  the maximum value of any key in the input
     */
    public Coding(int n, int m) {
        reset(n, m);
    }

    /**
     * Resets this object so that the encoding/decoding functions work for a new instance.
     * @param n  the number of elements in the input
     * @param m  the maximum value of any key in the input
     */
    public void reset(int n, int m) {
        this.n = n;
        this.m = m;
    }

    /**
     * Encoding function.
     * @param x  number that should be encoded
     * @param i  index of the number x
     * @return double  the wavelength that represents x
     */
    public double encode(int x, int i) {
        return (x + (i+1)/(double)(n+1)) /(double)(m+1) * (MAX_WL - MIN_WL) + MIN_WL;
    }

    /**
     * Decoding function.
     * @param wl  the wavelength that should be decoded
     * @return int  the number x that is represented by the wavelength wl
     */
    public int decode(double wl) {
        return (int)((wl - MIN_WL) * (m+1) / (MAX_WL - MIN_WL));
    }

    /** The minimum wavelength that is used. */
    private static final double MIN_WL = 380;

    /** The maximum wavelength that is used. */
    private static final double MAX_WL = 780;

    /** The number of elements in the input. */
    private int n;

    /** The maximum value of any key in the input. */
    private int m;
}
