/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 11.05.2004
 * Time: 23:50:24
 */

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.event.*;

/**
 * A dialog that allows the user to select the parameters for a new instance.
 */
public class InputDialog extends JDialog {

    /**
     * The constructor.
     * @param owner  the main application frame
     * @param n  the number of elements in the last instance
     * @param m  the maximum value in the last instance
     */
    public InputDialog(Frame owner, int n, int m) {
        super(owner, "Rainbow Sort", true);

        // create a slider for the number of items
        nSlider = new JSlider( JSlider.HORIZONTAL, 0, 500, n );
        nSlider.setMajorTickSpacing(50);
        nSlider.setMinorTickSpacing(10);
        nSlider.setPaintTicks(true);
        nSlider.setPaintLabels(true);
        nSlider.setPreferredSize( new Dimension(500,50) );
        JPanel nPanel = new JPanel();
        nPanel.add( new JLabel("number of items") );
        nPanel.add( nSlider );

        // create a slider for the maximum value
        mSlider = new JSlider( JSlider.HORIZONTAL, 0, 1000, m );
        mSlider.setMajorTickSpacing(100);
        mSlider.setMinorTickSpacing(20);
        mSlider.setPaintTicks(true);
        mSlider.setPaintLabels(true);
        mSlider.setPreferredSize( new Dimension(500,50) );
        JPanel mPanel = new JPanel();
        mPanel.add( new JLabel("maximum value") );
        mPanel.add( mSlider );

        // add both sliders to a panel
        JPanel dataPanel = new JPanel( new BorderLayout() );
        dataPanel.add( nPanel, BorderLayout.NORTH );
        dataPanel.add( mPanel, BorderLayout.SOUTH );

        // add OK/cancel-buttons to another panel
        JButton okayButton = new JButton("OK");
        ActionListener okayButtonListener = new ActionListener () {
            public void actionPerformed(ActionEvent event) {
                okay();
            }
        };
        okayButton.addActionListener( okayButtonListener );

        JButton cancelButton = new JButton("Cancel");
        ActionListener cancelButtonListener = new ActionListener () {
            public void actionPerformed(ActionEvent event) {
                cancel();
            }
        };
        cancelButton.addActionListener( cancelButtonListener );
        JPanel ctrlPanel = new JPanel();
        ctrlPanel.add( okayButton );
        ctrlPanel.add( cancelButton );

        // add all panels
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(dataPanel, BorderLayout.NORTH);
        getContentPane().add(ctrlPanel, BorderLayout.SOUTH);

        pack();
    }

    /**
     * Cancels the dialog if the window is closed.
     */
    protected void processWindowEvent(WindowEvent e) {
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
            cancel();
		else
            super.processWindowEvent(e);
    }

    /**
     * Returns true iff the dialog has been closed by pressing the OK-button.
     */
    public boolean isOkay() { return okay; }

    /**
     * Returns the value of the slider for the number of elements.
     */
    public int getN() { return nSlider.getValue(); }

    /**
     * Returns the value of the slider for the maximum value.
     */
    public int getM() { return mSlider.getValue(); }

    /**
     * Cancels the dialog.
     */
    private void cancel() {
        okay = false;
        dispose();
    }

    /**
     * Closes the dialog and stores the fact that the OK-button has been pressed.
     */
    private void okay() {
        okay = true;
        dispose();
    }

    private JSlider nSlider;
    private JSlider mSlider;
    private boolean okay;
}
