/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 11.05.2004
 * Time: 15:16:28
 */

import java.awt.*;

/**
 * A rgb-color. Provides a conversion from wavelength to rgb.
 */
public class MyColor {
    /**
     * The default constructor.
     */
    public MyColor() {
    }

    /**
     * Constructs a rgb-color according to a given wavelength.
     */
    public MyColor(int waveLength) {
        wavelengthToRGB(waveLength);
    }

    /**
     * Returns a jawa.awt.Color object that represents this color.
     */
    public Color getColor(){
        return new Color(r,g,b);
    }

    /**
     * Adds a given color to this color.
     */
    public void add(MyColor color) {
        r = Math.min( intensitMax, r + color.r );
        g = Math.min( intensitMax, g + color.g );
        b = Math.min( intensitMax, b + color.b );
    }

    /**
     * Returns true iff this color is a dark color.
     */
    public boolean isDark() {
        if (r + g + b > intensitMax * 2) return false;
        double factor = 4/(double)5;
        return ((r < intensitMax * factor) && (g < intensitMax * factor) && (b < intensitMax * factor));
    }

    /**
     * Returns the minimum wavelength that can be handled.
     */
    public static int minWavelength() {
        return 380;
    }

    /**
     * Returns the maximum wavelength that can be handled.
     */
    public static int maxWavelength() {
        return 780;
    }

    /**
     * Computes the rgb values of the given wavelength.
     */
    private void wavelengthToRGB(int wave){
        double red,blue,green;
        double wavel=(double)wave;
        double factor = 0.0;
        int z=0;
        if ((wavel>=380)&&(wavel<=439)) z=1;
        if ((wavel>=440)&&(wavel<=489)) z=2;
        if ((wavel>=490)&&(wavel<=509)) z=3;
        if ((wavel>=510)&&(wavel<=579)) z=4;
        if ((wavel>=580)&&(wavel<=644)) z=5;
        if ((wavel>=645)&&(wavel<=780)) z=6;
        if ((wavel>=380)&&(wavel<=419)) factor=0.3+0.7*(wavel-380.0)/(420.0-380.0);
        if ((wavel>=420)&&(wavel<=700)) factor=1.0;
        if ((wavel>=701)&&(wavel<=780)) factor=0.3+0.7*(780.0-wavel)/(780.0-700.0);

        switch (z) {
            case 1: {
                red = -(wavel-440.0)/(440.0-380.0);
                green = 0.0;
                blue = 1.0;
                r = adjust(red,factor);
                g = adjust(green,factor);
                b = adjust(blue,factor);
                break;
            }
            case 2: {
                red = 0.0;
                green = (wavel-440.0)/(490.0-440.0);
                blue = 1.0;
                r = adjust(red,factor);
                g = adjust(green,factor);
                b = adjust(blue,factor);
                break;
            }
            case 3: {
                red = 0.0;
                green = 1.0;
                blue = -(wavel-510.0)/(510.0-490.0);
                r = adjust(red,factor);
                g = adjust(green,factor);
                b = adjust(blue,factor);
                break;
            }
            case 4: {
                red = (wavel-510.0)/(580.0-510.0);
                green = 1.0;
                blue = 0.0;
                r = adjust(red,factor);
                g = adjust(green,factor);
                b = adjust(blue,factor);
                break;
            }
            case 5: {
                red = 1.0;
                green = -(wavel-645.0)/(645.0-580.0);
                blue = 0.0;
                r = adjust(red,factor);
                g = adjust(green,factor);
                b = adjust(blue,factor);
                break;
            }
            case 6: {
                red = 1.0;
                green = 0.0;
                blue = 0.0;
                r = adjust(red,factor);
                g = adjust(green,factor);
                b = adjust(blue,factor);
                break;
            }
            default: {
                red = 0.0;
                green = 0.0;
                blue = 0.0;
                break;
            }
        }
    }

    private int adjust(double col, double fact){
        double wert;
        if (col==0) return 0;
        else {
            wert = Math.round(intensitMax*Math.pow(col*fact,gamma));
            //System.out.println(wert);
            //System.out.println(intensitMax*Math.pow(col*fact,gamma));
            return (int) wert;
        }
    }


    private int r=0;
    private int g=0;
    private int b=0;
    final double gamma = 0.8;
    final int intensitMax = 255;
}
