/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 11.05.2004
 * Time: 12:57:41
 */

import javax.swing.*;
import java.util.ArrayList;
import java.util.Collections;

/**
 * Deals with the output of Rainbow Sort.
 */
public class Output {
    /**
     * The constructor.
     * @param coding  the coding object in order to be able to decode incoming wavelengths
     * @param table  the output table in order to be able to add decoded numbers to the output
     */
    public Output( Coding coding, JTable table ) {
        this.coding = coding;
        this.table = table;
        this.cache = new ArrayList();
        reset();
    }

    /**
     * Resets this output object in order to be able to deal with a new instance.
     */
    public void reset() {
        index = 0;
    }

    /**
     * Adds a wavelength that has arrived during the current time step to the cache.
     */
    public void addWavelength(double wl) {
        cache.add( new Double(wl) );
    }

    /**
     * Clears the cache after each time step.
     * Before the cache is cleared, it is sorted.
     * Then, the contents is decoded and added to the output table.
     */
    public void clearCache() {
        Collections.sort( cache );
        for (int i = cache.size()-1; i>=0; i--) {
            double wl = ((Double) cache.get(i) ).doubleValue();
            int number = coding.decode(wl);
            table.setValueAt( "" + number, 2, index++ );
            table.repaint();
        }
        cache.clear();
    }

    private int index;
    private Coding coding;
    private JTable table;
    private ArrayList cache;
}
