/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 10.05.2004
 * Time: 19:18:10
 */

/**
 * A prism.
 */
public class Prism {
    /**
     * The constructor.
     * @param a  a vertex of the prism (triangle)
     * @param b  a vertex of the prism (triangle)
     * @param c  a vertex of the prism (triangle)
     */
    public Prism(Vector2D a, Vector2D b, Vector2D c) {
        this.a = a;
        this.b = b;
        this.c = c;
    }

    /** Returns the vertex "a" of the prism. */
    public Vector2D a() {return a;}

    /** Returns the vertex "b" of the prism. */
    public Vector2D b() {return b;}

    /** Returns the vertex "c" of the prism. */
    public Vector2D c() {return c;}

    /**
     * Returns the point of intersection between a given line and the prism.
     * @param start  start point of the given line
     * @param end  end point of the given line
     * @param edge  a vector that represents the edge of the prism that intersects the given line
     *             (not read, but set by the method)
     * @return Vector2D  the point of intersection or null if there is no intersection
     */
    public Vector2D getIntersection(Vector2D start, Vector2D end, Vector2D edge) {
        Vector2D result;
        if ((result = Vector2D.getIntersectionOfTwoLines(start, end, a, b)) != null) {
            edge.set(a).subtract(b);
            return result;
        }
        if ((result = Vector2D.getIntersectionOfTwoLines(start, end, a, c)) != null) {
            edge.set(a).subtract(c);
            return result;
        }
        if ((result = Vector2D.getIntersectionOfTwoLines(start, end, b, c)) != null) {
            edge.set(b).subtract(c);
            return result;
        }
        return null;
    }

    /**
     * Returns the refraction index for the given wavelength.
     * Here, just a linear interpolation is implemented.
     */
    public double getRefractionIndex(double wl) {
        return (MAX_WL - wl) / (MAX_WL - MIN_WL) * (MAX_REFRACT - MIN_REFRACT) + MIN_REFRACT;
    }


    private static final double MIN_WL = 380;
    private static final double MAX_WL = 780;
    private static final double MIN_REFRACT = 1.2;
    private static final double MAX_REFRACT = 1.6;

    private Vector2D a, b, c;
}
