/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 10.05.2004
 * Time: 12:55:26
 */

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.*;
import java.awt.*;
import java.util.*;

/**
 * The main class of Rainbow Sort.
 *
 * @author Dominik Schultes
 * @version 1.0
 */
public class RainbowSort extends JFrame implements ChangeListener {
    /**
     * The constructor.
     */
    public RainbowSort() {
        super("Rainbow Sort");

        // input
        setInput();

        // gui
        createGUI();

        // create scene and rays
        createScene();
        rays = new Rays(scene);
        addRays();

        // add canvas
        canvas = new MyCanvas(CANVAS_WIDTH, CANVAS_HEIGHT, MAX_COORD_X, MAX_COORD_Y, rays, scene);
        getContentPane().add(canvas, BorderLayout.SOUTH);

        pack();
        setVisible(true);
    }

    /**
     * Creates the GUI.
     */
    private void createGUI() {
        getContentPane().setLayout(new BorderLayout());

        // CONTROL PANEL
        // start button
        JPanel controlPanel = new JPanel();
        startButton = new JButton("Start");
        ActionListener startButtonListener = new ActionListener () {
            public void actionPerformed(ActionEvent event) {
                pressStartButton();
            }
        };
        startButton.addActionListener( startButtonListener );
        startButton.setPreferredSize( new Dimension( 100, startButton.getPreferredSize().height) );

        // "new instance" button
        newInputButton = new JButton("New Instance");
        ActionListener newButtonListener = new ActionListener () {
            public void actionPerformed(ActionEvent event) {
                pressNewButton();
            }
        };
        newInputButton.addActionListener( newButtonListener );

        // delay slider
        sleeptimeSlider = new JSlider( JSlider.HORIZONTAL, 0, 200, 40 );
        sleeptimeSlider.setMajorTickSpacing(50);
        sleeptimeSlider.setMinorTickSpacing(10);
        sleeptimeSlider.setPaintTicks(true);
        sleeptimeSlider.setPaintLabels(true);
        sleeptimeSlider.addChangeListener(this);

        // add everything to the control panel
        controlPanel.add( startButton );
        controlPanel.add( newInputButton );
        controlPanel.add( new JLabel("        delay") );
        controlPanel.add( sleeptimeSlider );
        getContentPane().add( controlPanel, BorderLayout.NORTH);


        // add table
        table = new JTable( createTableModel() );
        table.setAutoResizeMode( JTable.AUTO_RESIZE_OFF );
        for (int i = 0; i < n; i++) table.getColumnModel().getColumn(i).setPreferredWidth( 35 );
        table.setDefaultRenderer( Object.class, new WavelengthTableCellRenderer( coding ) );
        table.setFocusable( false );
        String[][] rowHeaderNames = new String[3][1];
        rowHeaderNames[0][0] = "Input";
        rowHeaderNames[1][0] = "Wavelength";
        rowHeaderNames[2][0] = "Output";
        String[] rowHeaderColName = {""};
        JTable rowHeader = new JTable( rowHeaderNames, rowHeaderColName );
        rowHeader.setAutoResizeMode( JTable.AUTO_RESIZE_OFF );
        rowHeader.setBackground( table.getTableHeader().getBackground() );
        rowHeader.setFocusable( false );
        JScrollPane scrollPane = new JScrollPane( table, JScrollPane.VERTICAL_SCROLLBAR_NEVER,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setPreferredSize( new Dimension(CANVAS_WIDTH, 82) );
        scrollPane.setRowHeaderView( rowHeader );
        scrollPane.getRowHeader().setPreferredSize( new Dimension(75,82) );
        getContentPane().add(scrollPane, BorderLayout.CENTER);
    }

    /**
     * Creates a new SimpleTableModel.
     * The first row contains the input data. The second and third rows are empty.
     */
    private SimpleTableModel createTableModel() {
        String[][] tableData = new String[3][n];
        for (int j=0; j<n; j++) {
            tableData[0][j] = new String("" + inputData[j]);
        }
        for (int i=1; i<=2; i++) {
            for (int j=0; j<n; j++) {
                tableData[i][j] = new String("");
            }
        }
        return new SimpleTableModel( tableData );
    }

    /**
     * Terminates the application when the window is closed.
     */
    protected void processWindowEvent(WindowEvent e) {
        if (e.getID() == WindowEvent.WINDOW_CLOSING)
            System.exit(0);
		else
            super.processWindowEvent(e);
    }

    /**
     * Updates the delay of the simulation according to the value of the slider.
     */
    public void stateChanged(ChangeEvent e) {
        if (simulator != null) simulator.setTimestepReal( sleeptimeSlider.getValue() );
    }

    /**
     * Controls the simulation.
     */
    public void pressStartButton() {
        if (startButton.getText().equals("Start")) {
            // start the simulation
            startButton.setText("Stop");
            newInputButton.setEnabled(false);
            startSim();
        }
        else if (startButton.getText().equals("Stop")) {
            // stop the simulation
            startButton.setText("Continue");
            newInputButton.setEnabled(true);
            simulator.hold();
        }
        else if (startButton.getText().equals("Continue")) {
            // continue the simulation
            startButton.setText("Stop");
            newInputButton.setEnabled(false);
            simulator.goOn();
        }
        else if (startButton.getText().equals("Reset")) {
            // reset the simulation
            startButton.setText("Start");
            resetSim();
        }
    }

    /**
     * Creates a new instance.
     * The user can select the parameters n (number of elements) and m (maximum value).
     * The new instance is generated randomly.
     */
    public void pressNewButton() {
        InputDialog inputDialog = new InputDialog( this, n, m );
        inputDialog.show();
        if ( inputDialog.isOkay() ) {
            n = inputDialog.getN();
            m = inputDialog.getM();
            setRandomInput();
            table.setModel( createTableModel() );
            for (int i = 0; i < n; i++) table.getColumnModel().getColumn(i).setPreferredWidth( 35 );
            coding.reset(n, m);
            resetSim();
            startButton.setText("Start");
        }
    }

    /**
     * Creates a first instance when the application is started.
     */
    private void setInput() {
        n = 15;
        m = 40;
        setRandomInput();
        coding = new Coding(n, m);
    }

    /**
     * Creates a random instance according to the parameters n (number of elements) and m (maximum value).
     */
    private void setRandomInput() {
        Random random = new Random();
        inputData = new int[n];
        for (int i=0; i<n; i++) {
            inputData[i] = random.nextInt(m+1);
        }
    }

    /**
     * Creates the scene.
     * Here, one prism and one detector is created.
     */
    private void createScene() {
        output = new Output( coding, table );
        scene = new Scene( output );
        scene.addPrism( new Prism( new Vector2D(200,450),
                                   new Vector2D(600,450),
                                   new Vector2D(400,150) ) );
        scene.addDetector( new Detector( new Vector2D(650,75), new Vector2D(950,525) ) );
    }

    /**
     * Creates the initial rays.
     * Here, one ray is generated that consists of the wavelengths that represent the input data.
     */
    private void addRays() {
        Ray ray = new Ray(new Vector2D(10,500), new Vector2D(2, -1.5, true));
        for (int i=0; i<n; i++) {
            double wl = coding.encode(inputData[i], i);
            ray.addWavelength( wl );
            table.setValueAt( "" + (int)wl, 1, i);
        }
        rays.add(ray);
    }

    /**
     * Starts the simulation.
     */
    private void startSim() {
        simulator = new Simulator(rays, 2, sleeptimeSlider.getValue(), canvas, this);
        simulator.start();
    }

    /**
     * Terminates the simulation.
     */
    public void terminateSim() {
        simulator = null;
        checkResult();
        startButton.setText("Reset");
        newInputButton.setEnabled(true);
    }

    /**
     * Resets the simulation.
     */
    private void resetSim() {
        for (int i=0; i<n; i++) {
            table.setValueAt("", 2, i);
        }
        output.reset();
        rays.clear();
        addRays();
        repaint();
    }

    /**
     * Checks if the result of Rainbow Sort is sorted and complete.
     * Prints the test result to std out.
     */
    private void checkResult() {
        System.out.print("Check (sorted) ");
        boolean passed = true;
        for (int i=1; i<n; i++) {
            try {
                int x = Integer.parseInt( (String)table.getValueAt(2, i-1) );
                int y = Integer.parseInt( (String)table.getValueAt(2, i) );
                if (x < y) {
                    System.out.print(i + " ");
                    passed = false;
                }
            } catch (Exception e) {
                System.out.print(i + " ");
                passed = false;
            }
        }
        if (passed)
            System.out.println("passed.");
        else
            System.out.println("failed !");

        if (passed) {
            System.out.print("Check (complete) ");
            ArrayList sortedInput = new ArrayList();
            for (int i=0; i<n; i++) {
                sortedInput.add( new Integer(inputData[i]) );
            }
            Collections.sort( sortedInput );
            for (int i=0; i<n; i++) {
                if ( ((Integer)sortedInput.get(n-1-i)).intValue() !=
                        Integer.parseInt( (String)table.getValueAt(2, i) ) ) {
                    System.out.println(i + " failed !");
                    return;
                }
            }
            System.out.println("passed.");
        }
    }


    private static final int CANVAS_WIDTH = 600;
    private static final int CANVAS_HEIGHT = 450;
    private static final double MAX_COORD_X = 1000;
    private static final double MAX_COORD_Y = 750;

    private int n;
    private int m;
    private int[] inputData;

    private JTable table;
    private MyCanvas canvas;
    private JButton startButton;
    private JButton newInputButton;
    private JSlider sleeptimeSlider;

    private Rays rays;
    private Simulator simulator;
    private Coding coding;
    private Scene scene;
    private Output output;


    /**
     * The main method. Creates a RainbowSort object.
     */
    public static void main(String[] args) {
        new RainbowSort();
    }
}
