/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 10.05.2004
 * Time: 14:53:46
 */

import java.awt.*;
import java.awt.image.MemoryImageSource;
import java.util.ArrayList;

/**
 * Encapsulates all rays of the simulation.
 */
public class Rays extends ArrayList {
    /**
     * The constructor.
     * @param scene  the scene (prism, detector) in order to be able to compute intersections with the scene
     */
    public Rays(Scene scene) {
        this.scene = scene;
    }

    /**
     * Draws all rays.
     */
    public void draw(MyCanvas canvas, Graphics g) {
        for (int i=0; i<size(); i++) {
            ((Ray)get(i)).draw(canvas, g);
        }
    }

    /**
     * Returns true iff there is at least one active ray.
     */
    public boolean isActive() {
        for (int i=0; i<size(); i++) {
            if (((Ray)get(i)).isActive()) return true;
        }
        return false;
    }

    /**
     * Traces all rays for the given time interval.
     */
    public void trace(double time) {
        scene.resetNewRays();
        for (int i=0; i<size(); i++) {
            Ray ray = (Ray)get(i);
            if (! ray.isActive() ) continue;
            if (! scene.isIntersectingPrism(ray, time)) {
                if (! scene.isIntersectingDetector(ray, time)) {
                    ray.moveStraight(time);
                }
            }
        }
        scene.getOutput().clearCache();
        addAll( scene.getNewRays() );
    }

    private Scene scene;
}
