/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 10.05.2004
 * Time: 17:05:49
 */

/**
 * The simulator that controls the procedure of the simulation.
 */
public class Simulator extends Thread {
    /**
     * The constructor.
     * @param rays  all rays that are simulated
     * @param timestepSim  length of a time step in the simulated time
     * @param timestepReal  length of a time step in real time
     * @param canvas  canvas where everything is drawn
     * @param rainbowSort  main object in order to be able to report the end of the simulation
     */
    public Simulator(Rays rays, double timestepSim, int timestepReal, MyCanvas canvas, RainbowSort rainbowSort) {
        this.rays = rays;
        this.timestepSim = timestepSim;
        this.timestepReal = timestepReal;
        this.canvas = canvas;
        this.rainbowSort = rainbowSort;
        this.hold = false;
    }

    /**
     * Holds the simulation.
     */
    public void hold() { hold = true; }

    /**
     * Continues the simulation.
     */
    public void goOn() { hold = false; }

    /**
     * Sets the length of a time step in real time.
     */
    public void setTimestepReal(int timestepReal) {
        this.timestepReal = timestepReal;
    }

    /**
     * Runs the simulation.
     */
    public void run() {
        while ( rays.isActive() ) {

            rays.trace(timestepSim); // trace all rays

            canvas.repaint(); // draw everything

            // sleep for a while
            try {
                sleep(timestepReal);
            } catch (InterruptedException e) {
            }
            while (hold) {
                // wait until the user presses continue
                try {
                    sleep(50);
                } catch (InterruptedException e) {
                }
            }
        }
        System.out.println("Simulation terminated.");
        rainbowSort.terminateSim(); // report the end of the simulation
    }

    private Rays rays;
    private double timestepSim;
    private int timestepReal;
    private MyCanvas canvas;
    private RainbowSort rainbowSort;
    private boolean hold;
}
