/*
 * Created by IntelliJ IDEA.
 * User: Dominik
 * Date: 11.05.2004
 * Time: 21:50:11
 */

import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.*;
import java.awt.*;

/**
 * A TableCellRenderer that is used for the input/output table in order to
 * color the cells according to the corresponding wavelengths.
 */
public class WavelengthTableCellRenderer extends DefaultTableCellRenderer {
    /**
     * The constructor.
     */
    public WavelengthTableCellRenderer(Coding coding) {
        this.coding = coding;
    }

    /**
     * Overwrites the default method in order to color the cells.
     */
    public Component getTableCellRendererComponent
            (JTable table, Object value, boolean selected, boolean focused, int row, int column) {

        setEnabled(table == null || table.isEnabled());

        try {
            if (row == 1) { // second row that contains the wavelengths
                int wl = Integer.parseInt( value.toString() );
                if ((wl >= MyColor.minWavelength()) && (wl <= MyColor.maxWavelength())) {
                    // set the color according to the wavelength
                    MyColor color = new MyColor(wl);
                    setBackground( color.getColor() );
                    if ( color.isDark() )
                        setForeground( Color.WHITE );
                    else
                        setForeground( Color.BLACK );
                }
                else {
                    setBackground(null);
                    setForeground(null);
                }
            }
            else { // first or third row that contains the unsorted resp. sorted keys
                int x = Integer.parseInt( value.toString() );
                // set the color according to the corresponding wavelength
                MyColor color = new MyColor( (int)coding.encode( x, 0 ) );
                setBackground( color.getColor() );
                if ( color.isDark() )
                    setForeground( Color.WHITE );
                else
                    setForeground( Color.BLACK );
            }
        } catch (NumberFormatException e) {
            setBackground(null);
            setForeground(null);
        }


        super.getTableCellRendererComponent(table, value, selected, focused, row, column);

        return this;
    }

    private Coding coding;
}
